@@ -1,5 +1,5 @@
 ;;  -*-  indent-tabs-mode:nil; coding: utf-8 -*-
-;;  Copyright (C) 2013,2014,2015,2016,2017,2018
+;;  Copyright (C) 2013,2014,2015,2016,2017,2018,2019
 ;;      "Mu Lei" known as "NalaGinrut" <NalaGinrut@gmail.com>
 ;;  Artanis is free software: you can redistribute it and/or modify
 ;;  it under the terms of the GNU General Public License and GNU
@@ -41,6 +41,7 @@
   #:use-module (ice-9 control)
   #:use-module (web http)
   #:use-module (web request)
+  #:use-module (web response)
   #:use-module (web uri)
   #:use-module ((rnrs)
                 #:select (get-bytevector-all
@@ -51,18 +52,14 @@
                           get-string-all))
   #:export (regexp-split
             hash-keys cat bv-cat get-global-time sanitize-response
-
-            build-response write-response get-local-time string->md5 unsafe-random
-            uri-encode uri-decode response-version response-code response-connection
-            request-headers response-port write-response-body read-request request-uri
-            request-method request-content-length request-port read-request-body
-            response-content-length get-file-ext get-global-date get-local-date
+            get-local-time string->md5 unsafe-random parse-date write-date
+            get-file-ext get-global-date get-local-date
             string-substitute nfx static-filename remote-info seconds-now local-time-stamp
-            parse-date write-date make-expires export-all-from-module!
+            make-expires export-all-from-module!
             alist->hashtable expires->time-utc local-eval-string
             time-expired? valid-method? mmap munmap get-random-from-dev
             string->byteslist string->sha-1 string->sha-224 string->sha-256 string->sha-384
-            string->sha-512 list-slice bv-slice uni-basename
+            string->sha-512 list-slice bv-slice uni-basename detect-type-name
             checkout-the-path make-string-template guess-mime prepare-headers
             new-stack new-queue stack-slots queue-slots stack-pop! stack-push!
             stack-top stack-empty? queue-out! queue-in! queue-head queue-tail
@@ -72,17 +69,16 @@
             keyword->string range oah->handler oah->opts string->keyword
             alist->klist alist->kblist is-hash-table-empty?
             symbol-downcase symbol-upcase normalize-column run-before-run!
-            sxml->xml-string run-after-request! run-before-response!
-            make-pipeline HTML-entities-replace eliminate-evil-HTML-entities
-            generate-kv-from-post-qstr handle-proper-owner
+            sxml->xml-string run-after-request! run-before-response! run-when-DB-init! run-when-sigint-hook
+            run-when-sigint! make-pipeline HTML-entities-replace eliminate-evil-HTML-entities
+            generate-kv-from-post-qstr handle-proper-owner run-after-websocket-handshake!
             generate-data-url verify-ENTRY exclude-dbd
             draw-expander remove-ext scan-app-components cache-this-route!
             dump-route-from-cache generate-modify-time delete-directory
             handle-existing-file check-drawing-method DEBUG
             subbv->string subbv=? bv-read-line bv-read-delimited put-bv
             bv-u8-index bv-u8-index-right build-bv-lookup-table filesize
-            plist-remove gen-migrate-module-name try-to-load-migrate-cache
-            flush-to-migration-cache gen-local-conf-file with-dbd
+            plist-remove gen-migrate-module-name gen-local-conf-file with-dbd
             call-with-sigint define-box-type make-box-type unbox-type
             ::define did-not-specify-parameter colorize-string-helper
             colorize-string WARN-TEXT ERROR-TEXT REASON-TEXT
@@ -94,8 +90,38 @@
             artanis-log exception-from-client exception-from-server render-sys-page
             bv-copy/share bv-backward artanis-list-matches get-syspage
             artanis-sys-response char-predicate handle-upload is-valid-table-name?
-            is-guile-compatible-server-core?)
-  #:re-export (the-environment))
+            is-guile-compatible-server-core? positive-integer? negative-integer?
+            io-exception:peer-is-shutdown? io-exception:out-of-memory?
+            out-of-system-resources? allow-long-live-connection?)
+  #:re-export (the-environment
+               utf8->string
+               bytevector?
+
+               uri-decode
+               uri-encode
+               uri-query
+               uri-path
+
+               build-response
+               write-response
+               write-response-body
+
+               response-version
+               response-code
+               response-connection
+               response-port
+               write-response-body
+               response-content-length
+               read-request
+               read-request-body
+               request-uri
+               request-headers
+               request-method
+               request-content-length
+               request-port))
+
+(define parse-date (@@ (web http) parse-date))
+(define write-date (@@ (web http) write-date))
 
 ;; There's a famous rumor that 'urandom' is safer, so we pick it.
 (define* (get-random-from-dev #:key (length 8) (uppercase #f))
@@ -107,26 +133,6 @@
             (string-upcase str)
             str)))))
 
-(define uri-decode (@ (web uri) uri-decode))
-(define uri-encode (@ (web uri) uri-encode))
-(define parse-date (@@ (web http) parse-date))
-(define write-date (@@ (web http) write-date))
-(define build-response (@ (web response) build-response))
-(define write-response (@ (web response) write-response))
-(define response-version (@ (web response) response-version))
-(define response-code (@ (web response) response-code))
-(define response-connection (@ (web response) response-connection))
-(define response-port (@ (web response) response-port))
-(define write-response-body (@ (web response) write-response-body))
-(define response-content-length (@ (web response) response-content-length))
-(define read-request (@ (web request) read-request))
-(define read-request-body (@ (web request) read-request-body))
-(define request-uri (@ (web request) request-uri))
-(define request-headers (@ (web request) request-headers))
-(define request-method (@ (web request) request-method))
-(define request-content-length (@ (web request) request-content-length))
-(define request-port (@ (web request) request-port))
-
 (define-syntax-rule (local-eval-string str e)
   (local-eval
    (call-with-input-string (format #f "(begin ~a)" str) read)
@@ -205,28 +211,27 @@
   (hash-map->list (lambda (k v) k) ht))
 
 ;; WARN: besure that you've already checked the file exists before!!!
-(define* (cat file/port #:optional (out (current-output-port)))
+(define* (cat filename #:optional (out (current-output-port)))
   (define get-string-all (@ (rnrs io ports) get-string-all))
-  (let ((str (if (port? file/port)
-                 (get-string-all file/port)
-                 (call-with-input-file file/port get-string-all))))
+  (let ((str (if (port? filename)
+                 (throw 'artanis-err 500 cat
+                        "BUG: Shouldn't be port here (~a)!" filename)
+                 (call-with-input-file filename get-string-all))))
     (if out
         (display str out)
         str)))
 
 ;; WARN: besure that you've already checked the file existtance before!!!
-(define* (bv-cat file/port #:optional (out (current-output-port)))
+(define* (bv-cat filename #:optional (out (current-output-port)))
   (define get-bytevector-all (@ (rnrs io ports) get-bytevector-all))
-  (let ((bv (if (port? file/port)
-                (get-bytevector-all file/port)
-                (call-with-input-file file/port get-bytevector-all))))
+  (let ((bv (if (port? filename)
+                (throw 'artanis-err 500 bv-cat
+                       "BUG: Shouldn't be port here (~a)!" filename)
+                (call-with-input-file filename get-bytevector-all))))
     (if out
         (display bv out)
         bv)))
 
-(define (string->md5 str)
-  (call-with-input-string str md5))
-
 ;; 35147 is the length of GPLv3 in bytes
 (define* (unsafe-random #:optional (n 35147))
   (random n (random-state-from-platform)))
@@ -277,8 +282,8 @@
 
 (define-syntax-rule (remote-info req)
   (if (get-conf '(server nginx))
-      (assoc-ref (request-headers req) 'x-real-ip)
-      (request-ip req)))
+      (or (assoc-ref (request-headers req) 'x-real-ip) "Unknown IP")
+      (or (request-ip req) "Unknown IP")))
 
 (define *methods-list* '(HEAD GET POST PUT PATCH DELETE))
 (define (allowed-method? method)
@@ -344,6 +349,16 @@
       (lp (cons (string->number (substring/shared str i (+ i step)) base) ret) (1+ i)))
      (else (lp ret (1+ i))))))
 
+(define (string->md5 str/bv)
+  (let ((in (cond
+             ((string? str/bv)
+              ((@ (rnrs) string->utf8) str/bv))
+             (((@ (rnrs) bytevector?) str/bv)
+              str/bv)
+             (else (throw 'artanis-err 500 string->sha-1
+                          "need string or bytevector!" str/bv)))))
+    (md5->string (md5 in))))
+
 (define (string->sha-1 str/bv)
   (let ((in (cond
              ((string? str/bv)
@@ -559,7 +574,7 @@
   (mime-guess (get-file-ext filename)))
 
 (define (bytevector-null? bv)
-  ((@ (rnrs bytevectors) bytevector=?) bv #u8()))
+  ((@ (rnrs bytevectors) bytevector=?) bv #vu8()))
 
 (define (generate-modify-time t)
   (get-local-date (cons (time-second t) (time-nanosecond t))))
@@ -696,6 +711,18 @@
 (define (run-before-run! proc)
   (add-hook! *before-run-hook* proc))
 
+(define (run-when-DB-init! proc)
+  (add-hook! *DB-conn-init-hook* proc))
+
+(define (run-after-websocket-handshake! proc)
+  (add-hook! *after-websocket-handshake-hook* proc))
+
+(define (run-when-sigint! proc)
+  (add-hook! *when-sigint-hook* proc))
+
+(define (run-when-sigint-hook)
+  (run-hook *when-sigint-hook*))
+
 ;; NOTE: For `pipeline' methodology, please read my post:
 ;; http://nalaginrut.com/archives/2014/04/25/oba-pipeline-style%21
 (define (make-pipeline . procs)
@@ -744,7 +771,7 @@
     (match lst
       ((k v) (list (key-converter k) ((current-encoder) v)))
       (else (throw 'artanis-err 500 generate-kv-from-post-qstr
-                   "Fatal! Can't be here!" lst))))
+                   "Fatal! Can't be here `~a'!" lst))))
   (define (-> x)
     (string-trim-both x (lambda (c) (member c '(#\sp #\: #\return)))))
   (map (lambda (x)
@@ -837,9 +864,8 @@
     (format port ";; Do not touch anything!!!~%")
     (format port ";; All things here should be automatically handled properly!!!~%"))
   (define route-cache (string-append (current-toplevel) "/tmp/cache/route.cache"))
-  (when (or (not (file-exists? route-cache))
-            (and (not url) (not meta))) ; for regenerating route cache
-    (format (artanis-current-output) "Route cache is missing, regenerating...~%")
+  (when (not (file-exists? route-cache))
+    (format (artanis-current-output) "Regenerating route cache ...~%")
     (call-with-output-file route-cache
       (lambda (port) (write-header port) (write '() port))))
   (when (and url meta)
@@ -1011,27 +1037,6 @@
    (else (throw 'artanis-err 500 gen-migrate-module-name
                 "Wrong parsing of module name, shouldn't be here!" f))))
 
-(define (try-to-load-migrate-cache name)
-  (let ((file (format #f "~a/tmp/cache/migration/~a.scm" (current-toplevel) name)))
-    (cond
-     ((file-exists? file) (load file))
-     (else
-      (format (artanis-current-output)
-              "[WARN] No cache for migration of `~a'~%" name)
-      (format (artanis-current-output)
-              "Run `art migrate up ~a', then try again!~%" name)))))
-
-(define (flush-to-migration-cache name fl)
-  (let ((file (format #f "~a/tmp/cache/migration/~a.scm" (current-toplevel) name)))
-    (when (file-exists? file) (delete-file file))
-    (call-with-output-file file
-      (lambda (port)
-        (format port "(define-~a~%" name)
-        (for-each
-         (lambda (ft) (format port "~2t~a~%" ft))
-         fl)
-        (format port "~2t)~%")))))
-
 (define (gen-local-conf-file)
   (format #f "~a/conf/artanis.conf" (current-toplevel)))
 
@@ -1069,7 +1074,9 @@
               (dynamic-wind
                 (lambda ()
                   (set! handler
-                    (sigaction SIGINT (lambda (sig) (throw 'interrupt)))))
+                    (sigaction SIGINT (lambda (sig)
+                                        (run-when-sigint-hook)
+                                        (throw 'interrupt)))))
                 thunk
                 (lambda ()
                   (if handler
@@ -1252,7 +1259,9 @@
 (define (get-addr)
   (let ((host (get-conf '(host addr)))
         (family (get-family)))
-    (inet-pton family host)))
+    (if (and host (not (string=? host "127.0.0.1")))
+        (inet-pton family host)
+        INADDR_LOOPBACK)))
 
 (define (request-path req)
   (uri-path (request-uri req)))
@@ -1341,18 +1350,20 @@
   (memq name *guile-compatible-server-core*))
 
 (define (render-sys-page blame-who? status request)
-  (define-syntax-rule (status->page s)
-    (format #f "~a.html" s))
   (artanis-log blame-who? status 'text/html #:request request)
-  (let* ((charset (get-conf '(server charset)))
+  (let* ((filename (format #f "~a.html" status))
+         (charset (get-conf '(server charset)))
          (mtime (generate-modify-time (current-time)))
+         (guile-compt-serv? (is-guile-compatible-server-core? (get-conf '(server engine))))
          (response
           (build-response #:code status
                           #:headers `((server . ,(get-conf '(server info)))
                                       (last-modified . ,mtime)
                                       (content-type . (text/html (charset . ,charset))))))
-         (body (syspage-show (status->page status))))
-    (if (is-guile-compatible-server-core? (get-conf '(server engine)))
+         (body (if (resources-collecting?)
+                   #vu8(0) ; Don't open any file since we don't have resources now.
+                   (syspage-show filename))))
+    (if guile-compt-serv?
         (values response body)
         (values response body 'exception))))
 
@@ -1485,3 +1496,32 @@
 (define is-valid-table-name?
   (lambda (name)
     (not (string-any invalid-char-set? name))))
+
+(define (positive-integer? x)
+  (and (integer? x) (positive? x)))
+
+(define (negative-integer? x)
+  (and (integer? x) (negative? x)))
+
+;; WARNING: Don't use = here, must use eqv? since it's not always numbers
+(define (io-exception:peer-is-shutdown? e)
+  (and (eq? (car e) 'system-error)
+       (let ((errno (system-error-errno e)))
+         (or (eqv? errno EPIPE) ; broken pipe
+             (eqv? errno EIO) ; write to a closed socket
+             (eqv? errno ECONNRESET))))) ; shutdown by peer
+
+;; WARNING: Don't use = here, must use eqv? since it's not always numbers
+(define (io-exception:out-of-memory? e)
+  (and (eq? (car e) 'system-error)
+       (let ((errno (system-error-errno e)))
+         (or (eqv? errno ENOMEM) ; no memory
+             (eqv? errno ENOBUFS)))))  ; no buffer could be allocated
+
+(define (out-of-system-resources? e)
+  (and (eq? (car e) 'system-error)
+       (let ((errno (system-error-errno e)))
+         (eqv? errno EMFILE)))) ; no more file could be opened
+
+(define (allow-long-live-connection?)
+  (> (get-conf '(server timeout)) 0))
